""" Manage the command line options to the form compiler executable """

from argparse import ArgumentParser
from os import path
from pytools import memoize

from dune.common.parametertree.parser import parse_ini_file


@memoize
def get_form_compiler_arguments():
    # define an argument parser.
    parser = ArgumentParser(description="Compile UFL files to PDELab C++ code", epilog="Please report bugs to dominic.kempf@iwr.uni-heidelberg.de")
    parser.add_argument("uflfile", type=str, nargs=1, help="the UFL file to compile")
    parser.add_argument("--driver-file", type=str, help="The filename for the generated driver header")
    parser.add_argument("--operator-file", type=str, help="The filename for the generated local operator header")
    parser.add_argument('--version', action='version', version='%(prog)s 0.1')
    parser.add_argument("--numerical-jacobian", action="store_true", help="use numerical jacobians (only makes sense, if uflpdelab for some reason fails to generate analytic jacobians)")
    parser.add_argument("--matrix-free", action="store_true", help="use iterative solver with matrix free jacobian application")
    parser.add_argument("--explicit-time-stepping", action="store_true", help="use explicit time stepping")
    parser.add_argument(
        "--exact-solution-expression",
        type=str,
        help="name of the exact solution expression in the ufl file")
    parser.add_argument(
        "--compare-dofs",
        type=str,
        help="maximal allowed maximum error of difference between degrees of freedom vectors of numerical solution and interpolation of exact solution (NOTE: requires --exact-solution-expression)"
    )
    parser.add_argument(
        "--compare-l2errorsquared",
        type=str,
        help="maximal allowed l2 error squared of difference between numerical solution and interpolation of exact solution (NOTE: requires --exact-solution-expression)"
    )
    parser.add_argument("--interactive", action="store_true", help="whether the optimization process should be guided interactively (also useful for debugging)")
    parser.add_argument("--print-transformations", action="store_true", help="print out dot files after ufl tree transformations")
    parser.add_argument("--print-transformations-dir", type=str, help="place where to put dot files (can be omitted)")
    parser.add_argument("--quadrature-order", type=int, help="Quadrature order used for all integrals.")
    parser.add_argument("--diagonal-transformation-matrix", action="store_true", help="set option if the jacobian of the transformation is diagonal (axiparallel grids)")
    parser.add_argument("--constant-transformation-matrix", action="store_true", help="set option if the jacobian of the transformation is constant on a cell")
    parser.add_argument("--ini-file", type=str, help="An inifile to use. A generated driver will be hard-coded to it, a [formcompiler] section will be used as default values to form compiler arguments (use snake case)")
    parser.add_argument("--timer", action="store_true", help="measure times")
    parser.add_argument("--opcounter", action="store_true", default=False, help="Count operations. Should only be used with yaspgrid. Timer should be set.")
    parser.add_argument("--project-basedir", type=str, help="The base (build) directory of the dune-perftool project")
    # TODO at some point this help description should be updated
    parser.add_argument("--sumfact", action="store_true", help="Use sumfactorization")
    parser.add_argument("--vectorize", action="store_true", help="whether to generate code with explicit vectorization")

    # Modify the positional argument to not be a list
    args = vars(parser.parse_args())

    # Delistify the uflfile parameter
    args["uflfile"] = args["uflfile"][0]

    # Turn any relative paths into absolute ones for consistency
    if args["driver_file"]:
        args["driver_file"] = path.abspath(args["driver_file"])
    if args["operator_file"]:
        args["operator_file"] = path.abspath(args["operator_file"])
    if args["ini_file"]:
        args["ini_file"] = path.abspath(args["ini_file"])

    # Return the argument dict. This result is memoized to turn all get_option calls into simple dict lookups.
    return args


_option_dict = {}
_arguments_read = False


def init_option_dict():
    """Add form compile arguments to options dict"""
    global _arguments_read
    if not _arguments_read:
        _option_dict.update(get_form_compiler_arguments())
        _arguments_read = True

        # Read arguments from the given inifile
        if _option_dict["ini_file"]:
            inifile = parse_ini_file(_option_dict["ini_file"])
            for key, value in inifile.get("formcompiler", {}).items():
                if key not in _option_dict or _option_dict[key] is None:
                    _option_dict[key] = value
                elif not _option_dict[key]:
                    # As `bool("0")` is True, we need to eval bools
                    if isinstance(_option_dict[key], bool):
                        _option_dict[key] = eval(value)
                    else:
                        _option_dict[key] = type(_option_dict[key])(value)


def set_option(key, value):
    """Add the key value pair to the options.

    If the key is already in the options dictionary its value will be
    overwritten.  Form compiler arguments will always be set before
    any other options.

    """
    # Make sure form compile arguments were read
    init_option_dict()

    _option_dict.update({key: value})


def get_option(key, default=None):
    try:
        __IPYTHON__
        return default
    except:
        """Return the value corresponding to key from option dictionary"""
        # Make sure form compile arguments were read
        init_option_dict()

        return _option_dict.get(key, default)


def option_switch(opt):
    def _switch():
        if get_option(opt):
            return opt
        else:
            return "default"
    return _switch
